@@ -1,7 +1,7 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 2010, 2013 Oracle and/or its affiliates.  All rights reserved.
+ * Copyright (c) 2010, 2017 Oracle and/or its affiliates.  All rights reserved.
  */
 
 typedef struct PgHdr PgHdr;
@@ -17,4 +17,8 @@ void sqlite3PCacheSetDefault(void);
 #ifdef SQLITE_TEST
 void sqlite3PcacheStats(int *a,int *b,int *c,int *d);
 #endif
+#ifdef SQLITE_ENABLE_MEMORY_MANAGEMENT
+int sqlite3PcacheReleaseMemory(int nReq);
+#endif
+
 
