@@ -1,7 +1,7 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 2002, 2013 Oracle and/or its affiliates.  All rights reserved.
+ * Copyright (c) 2002, 2017 Oracle and/or its affiliates.  All rights reserved.
  *
  */
 
@@ -63,12 +63,10 @@ public class IndexTest extends TxnTestCase {
 
     @Parameters
     public static List<Object[]> genParams() {
-       
-        return getTxnTypes(null, false);
+        return getTxnParams(null, false);
     }
     
     public IndexTest(String type){
-      
         initEnvConfig();
         txnType = type;
         isTransactional = (txnType != TXN_NULL);
